#	Skrypt do transpozycji zapisu kodu źródłowego dla konkretnej pieśni
#	o wskazaną licznę kwint.
#	W zmiennej songid należy podać identyfikator pieśni, np.
#	songid="S106MaryjaMatkaKosciola"; natomiast w zmiennej transQ należy
#	podać liczbę kwint, np. transQ=1;
#	Skrypt tylko pomiędzy \begin{songbody} i \end{songbody) wyszuka komendy
#	akordów a jeśli po notacji znajdzie suffix, to sprawdzi jego zgodność
#	a następnie podmieni komendę na odpowiednią z tonacji docelowej.
#	Podmieni także sygnaturę tonacji w poprzedzającym \begin{songhead},
#	newet, jeśli są zdublowane instrukcjami warunkowymi, np \ifnojahwe.
#
#	Uwaga! Tablica suffix aktualizuje się automatycznie z chords.sty
#	poprzez skanowanie wywołania makr \chminfamdef i \chmajfamdef
#	i aktualny plik chords.sty musi być dostępny w tym samym katalogu.
#
#	Przykładowe wywołanie:
#	awk -v songid="S222Akeda" -v transQ=1 -f transposition.awk spprekas.tex > spprekas1.tex
#
#	Uwaga! skrypt ma naturę pomocniczą a nie automatyczną i ma za zadanie
#	jedynie ułatwić przepisanie tonacji podstawowej wybranej pieśni, gdyż
#	jeszcze wiele pieśni ma różne tonacje w śpiewnikach włoskim i hiszpańskim.
#	Samemu należy skorygować odpowiednie zapisy w tps.tex oraz ocg.tex
#	aby pasowały do nowej tonacji. Warto też sprawdzić sensowność składu
#	akordów w nowej tonacji.
#
BEGIN{
#
# Nazwy akordów oraz ich indeksy na kole kwintowym; ujemne dla molowych
#
  chordno["Feses"]=1; chord[1]="Feses";
  chordno["Ceses"]=2; chord[2]="Ceses";
  chordno["Geses"]=3; chord[3]="Geses";
  chordno["Deses"]=4; chord[4]="Deses";
  chordno["Ases"]=5; chord[5]="Ases";
  chordno["Eses"]=6; chord[6]="Eses";
  chordno["Heses"]=7; chord[7]="Heses";
  chordno["Fes"]=8; chord[8]="Fes";
  chordno["Ces"]=9; chord[9]="Ces";
  chordno["Ges"]=10; chord[10]="Ges";
  chordno["Des"]=11; chord[11]="Des";
  chordno["As"]=12; chord[12]="As";
  chordno["Es"]=13; chord[13]="Es";
  chordno["B"]=14; chord[14]="B";
  chordno["F"]=15; chord[15]="F";
  chordno["C"]=16;chord[16]="C";
  chordno["G"]=17; chord[17]="G";
  chordno["D"]=18; chord[18]="D";
  chordno["A"]=19; chord[19]="A";
  chordno["E"]=20; chord[20]="E";
  chordno["H"]=21; chord[21]="H";
  chordno["Fis"]=22; chord[22]="Fis";
  chordno["Cis"]=23; chord[23]="Cis";
  chordno["Gis"]=24; chord[24]="Gis";
  chordno["Dis"]=25; chord[25]="Dis";
  chordno["Ais"]=26; chord[26]="Ais";
  chordno["Eis"]=27; chord[27]="Eis";
  chordno["His"]=28; chord[28]="His";
  chordno["Fisis"]=29; chord[29]="Fisis";
  chordno["Cisis"]=30; chord[30]="Cisis";
  chordno["Gisis"]=31; chord[31]="Gisis";
  chordno["Disis"]=32; chord[32]="Disis";
  chordno["Aisis"]=33; chord[33]="Aisis";
  chordno["Eisis"]=34; chord[34]="Eisis";
  chordno["Hisis"]=35; chord[35]="Hisis";
  chordno["feses"]=-1; chord[-1]="feses";
  chordno["ceses"]=-2; chord[-2]="ceses";
  chordno["geses"]=-3; chord[-3]="geses";
  chordno["deses"]=-4; chord[-4]="deses";
  chordno["ases"]=-5; chord[-5]="ases";
  chordno["eses"]=-6; chord[-6]="eses";
  chordno["heses"]=-7; chord[-7]="heses";
  chordno["fes"]=-8; chord[-8]="fes";
  chordno["ces"]=-9; chord[-9]="ces";
  chordno["ges"]=-10; chord[-10]="ges";
  chordno["des"]=-11; chord[-11]="des";
  chordno["as"]=-12; chord[-12]="as";
  chordno["es"]=-13; chord[-13]="es";
  chordno["b"]=-14; chord[-14]="b";
  chordno["f"]=-15; chord[-15]="f";
  chordno["c"]=-16;chord[-16]="c";
  chordno["g"]=-17; chord[-17]="g";
  chordno["d"]=-18; chord[-18]="d";
  chordno["a"]=-19; chord[-19]="a";
  chordno["e"]=-20; chord[-20]="e";
  chordno["h"]=-21; chord[-21]="h";
  chordno["fis"]=-22; chord[-22]="fis";
  chordno["cis"]=-23; chord[-23]="cis";
  chordno["gis"]=-24; chord[-24]="gis";
  chordno["dis"]=-25; chord[-25]="dis";
  chordno["ais"]=-26; chord[-26]="ais";
  chordno["eis"]=-27; chord[-27]="eis";
  chordno["his"]=-28; chord[-28]="his";
  chordno["fisis"]=-29; chord[-29]="fisis";
  chordno["cisis"]=-30; chord[-30]="cisis";
  chordno["gisis"]=-31; chord[-31]="gisis";
  chordno["disis"]=-32; chord[-32]="disis";
  chordno["aisis"]=-33; chord[-33]="aisis";
  chordno["eisis"]=-34; chord[-34]="eisis";
  chordno["hisis"]=-35; chord[-35]="hisis";
  i=0;
#
# Sekwencja dopasowań nazw akordów, poczynając od najdłuższych
#  
  chordname[++i]="Feses";
  chordname[++i]="Ceses";
  chordname[++i]="Geses";
  chordname[++i]="Deses";
  chordname[++i]="Heses";
  chordname[++i]="feses";
  chordname[++i]="ceses";
  chordname[++i]="geses";
  chordname[++i]="deses";
  chordname[++i]="heses";
  chordname[++i]="Fisis";
  chordname[++i]="Cisis";
  chordname[++i]="Gisis";
  chordname[++i]="Disis";
  chordname[++i]="Aisis";
  chordname[++i]="Eisis";
  chordname[++i]="Hisis";
  chordname[++i]="fisis";
  chordname[++i]="cisis";
  chordname[++i]="gisis";
  chordname[++i]="disis";
  chordname[++i]="aisis";
  chordname[++i]="eisis";
  chordname[++i]="hisis";
  chordname[++i]="Ases";
  chordname[++i]="Eses";
  chordname[++i]="ases";
  chordname[++i]="eses";
  chordname[++i]="Fes";
  chordname[++i]="Ces";
  chordname[++i]="Ges";
  chordname[++i]="Des";
  chordname[++i]="fes";
  chordname[++i]="ces";
  chordname[++i]="ges";
  chordname[++i]="des";
  chordname[++i]="Fis";
  chordname[++i]="Cis";
  chordname[++i]="Gis";
  chordname[++i]="Dis";
  chordname[++i]="Ais";
  chordname[++i]="Eis";
  chordname[++i]="His";
  chordname[++i]="fis";
  chordname[++i]="cis";
  chordname[++i]="gis";
  chordname[++i]="dis";
  chordname[++i]="ais";
  chordname[++i]="eis";
  chordname[++i]="his";
  chordname[++i]="As";
  chordname[++i]="Es";
  chordname[++i]="as";
  chordname[++i]="es";
  chordname[++i]="B";
  chordname[++i]="F";
  chordname[++i]="C";
  chordname[++i]="G";
  chordname[++i]="D";
  chordname[++i]="A";
  chordname[++i]="E";
  chordname[++i]="H";
  chordname[++i]="b";
  chordname[++i]="f";
  chordname[++i]="c";
  chordname[++i]="g";
  chordname[++i]="d";
  chordname[++i]="a";
  chordname[++i]="e";
  chordname[++i]="h";
#
#	Wczytanie suffiksów dla rodzin komend akordowych
#
  while(getline < "chords.sty" > 0){
    if($0~/\\chm.+famdef{[^}]+}/){
      if($0~/\\chmajfamdef{.+}/){
        majorminor=1
      }else{
        majorminor=-1
      }
      split($0,a,"{");
      gsub("}","",a[2]);
      suffix[a[2]]=majorminor;
      suffix[a[2]"vA"]=majorminor;
      suffix[a[2]"vB"]=majorminor;
    }
  }
  collect=0;
  if(!transQ){
    transQ=1;
    songid="S106MaryjaMatkaKosciola";
  }
}
/\\songid{/{
  split($0,a,"{");
  currentsongid=a[3];
  gsub("}","",currentsongid);
  thissong=currentsongid==songid;
  for(ii=1;ii<=collect;ii++){
    print(memory[ii,thissong]);
  }
  collect=0;
}
{ if($0~/\\begin{songhead}/){
    memory[++collect,0]=$0;
    elems=patsplit($0,a,/{[^}]*}/,sep);ll="";
    keytone=a[4];gsub("{","",keytone);gsub("}","",keytone);
    oldkey=chordno[keytone];
    newkey=oldkey>0?oldkey+transQ:oldkey-transQ;
    a[4]="{"chord[newkey]"}";
    for(ii=0;ii<=elems;ii++){ll=ll""sprintf("%s",a[ii]""sep[ii])};
    memory[collect,1]=ll;
  }else if($0~/\\begin{songbody}/){
      print;
      songbody=1;
  }else if($0~/\\end{songbody}/){
    print;
    songbody=0;
  }else if(thissong&&songbody){
    line=$0;
    i=1;
    do{
      findin=substr(line,i);
      position=index(findin,"\\");
      if(!position)break;
      i+=position;
      found=0;
      for(ii=1;ii<=70;ii++){
        n=chordname[ii];
        if(found)break;
        checkin=substr(line,i,length(n));
        if(n==checkin){
          found=1;
          ch=n;
          mm=chordno[ch];
          continue;
        }
        if(substr(line,i,1)=="t"){
          i++;
          checkin=substr(line,i,length(n));
          if(n!=checkin)continue;
          found=1;
          ch=n;
          mm=chordno[ch];
        }
      }
      if(!found)continue;
      name=checkin;
      j=i+length(name);
      found=0;
      findin=substr(line,j);
      match(findin,/^[A-Za-z]+/);
      findrest=substr(findin,RSTART,RLENGTH)
      if(RSTART){
        for(s in suffix){
          chno=suffix[s];
          if((findrest==s)&&(chno*mm>0)){
            found=1;
            break
          }
        }
      }else{
        found=1;
      }
      if(!found)continue;
      pre=substr(line,1,i-1);
      tochange=substr(line,i,j-i);
      post=substr(line,j);
      key=chordno[tochange];
      newkey=key>0?key+transQ:key-transQ;
      line=pre""chord[newkey]""post;
    }while(1);
    print(line);
  }else if(collect){
      memory[++collect,0]=$0;
      memory[collect,1]=$0;
  }else{
    print;
  }
}